library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_unsigned.all;
package my_package is

--------------------------------------------------------------
-- p=2**192 - 2**64 - 1
--------------------------------------------------------------
constant N: integer := 192; 
constant logN: integer := 8;
constant number: std_logic_vector(logN downto 0) := conv_std_logic_vector(n, logN+1);
constant module: std_logic_vector(N+1 downto 0) := 
  "00" &  x"FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFEFFFFFFFFFFFFFFFF";
constant two_module: std_logic_vector(N+1 downto 0) := 
  "01" &  x"FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFEFFFFFFFFFFFFFFFE";
constant minus_module: std_logic_vector(N+1 downto 0) := 
  "11" &  x"000000000000000000000000000000010000000000000001";  

------------------------------------------------------------
-- p = 2**32 - 387
--------------------------------------------------------------
--constant N: integer := 32; 
--constant logN: integer := 5;
--constant number: std_logic_vector(logN downto 0) := conv_std_logic_vector(n, logN+1);
--constant module: std_logic_vector(N+1 downto 0) := "00" & x"FFFFFE7D"; --  2**32 - 387
--constant two_module: std_logic_vector(N+1 downto 0) := "01" & x"FFFFFCFA"; -- (2**32 - 387)*2
--constant minus_module: std_logic_vector(N+1 downto 0) := "11" & x"00000183";  
--                                                   
--------------------------------------------------------------
-- p = 239
--------------------------------------------------------------
--constant N: integer := 8;
--constant logN: integer := 3;
--constant number: std_logic_vector(logN downto 0) := conv_std_logic_vector(n, logN+1);
--constant module: std_logic_vector(N+1 downto 0) := "0011101111"; -- 239
--constant two_module: std_logic_vector(N+1 downto 0) := "0111011110"; -- 2*239 = 478
--constant minus_module: std_logic_vector(N+1 downto 0) := "1100010001"; --2**9 - 239 (-239) 
--
--------------------------------------------------------------
-- p = 2*64 - 1 
--------------------------------------------------------------
--constant N: integer := 64;
--constant logN: integer := 7;
--constant number: std_logic_vector(logN downto 0) := conv_std_logic_vector(n, logN+1);
--constant module: std_logic_vector(N+1 downto 0) := 
--"001111111111111111111111111111111111111111111111111111111111111111"; --2**64 - 1
--constant two_module: std_logic_vector(N+1 downto 0) := 
--"011111111111111111111111111111111111111111111111111111111111111110"; --2*(2**64 -1) = 2**65 - 2
--constant minus_module: std_logic_vector(N+1 downto 0) := 
--"110000000000000000000000000000000000000000000000000000000000000001"; --2**64 + 1


--------------------------------------------------------------
-- funtion definitions
function init_lut (i: natural) return bit_vector;
function init_lut_adj (i: natural) return bit_vector;
function to_string (stdlogvect: std_logic_vector ) return string;

end my_package;

package body my_package is

    function init_lut (i: natural) return bit_vector is
      variable ini_lut: std_logic_vector(7 downto 0); 
	 begin
	   if i = 0 then
	     ini_lut := "01" & not module(i) & module(i) & "1010";
	   elsif (i < N+2) then
		    ini_lut := module(i) & not module(i) & not module(i) & module(i) & not module(i-1) & module(i-1) & "10";
		else
		    ini_lut := "01101010";
      end if;
		return(To_bitvector(ini_lut));
    end init_lut;

    function init_lut_adj (i: natural) return bit_vector is
      variable ini_lut: std_logic_vector(7 downto 0); 
	 begin
	   if i = 0 then
	     ini_lut := "01" & not module(i) & module(i) & "1010";
	   elsif (i < N+2) then
		    ini_lut := module(i) & not module(i) & not module(i) & module(i) & not module(i-1) & module(i-1) & "10";
		else
		    ini_lut := "01101010";
      end if;
		return(To_bitvector(ini_lut));
    end init_lut_adj;

    
    function to_string (stdlogvect: std_logic_vector) return string is
		  variable an_string: string(8 downto 1); 
        begin
				for i in stdlogvect'range loop 
				  if stdlogvect(i) = '0' then
						an_string(i+1) := '0';
				  elsif stdlogvect(i) = '1' then
						an_string(i+1) := '1';
				  else
						an_string(i+1) := 'X';
              end if;
				end loop;
				return an_string;
    end to_string;
end my_package;


